var gtxmcperm = function ($, settings) {
	var folders, // channel folders 
		$store = $('input[name=f_node_restriction]'), // location to write json data to
		nodeRestriction = JSON.parse($store.val()); // mapping between groups and nodes

	/**
	 * update the tree to correctly display "assign node" icons
	 */
	function updateAssignNodeIcons() {
		$('.assignNodes').each(function () {
			var $icon = $(this),
				groupId = $icon.data('groupid'),
				checked = $('input[name=usergroup_chk\\[' + groupId + '\\]]').attr('checked');
			if (checked === 'checked') {
				if (nodeRestriction[groupId] && nodeRestriction[groupId].length > 0) {
					$icon.fadeTo('fast', 1.0);
				} else {
					$icon.fadeTo('fast', 0.4);
				}
			} else {
				$icon.fadeOut('fast');
			}
		});
	}
	
	/**
	 * updates the restriction object with new values
	 * updates assign node icons
	 * and writes it to the store
	 */
	function store() {
		var groupId = $('#channels').data('groupid'),
			invisible = $('#channels').data('invisible'),
			restriction = [],
			checkboxes = $('#channels li input:checked:enabled'), // find checked checkboxes
			$cb,
			i;

		for (i = 0; i < invisible.length; i++) {
			restriction.push(invisible[i]);
		}

		if (checkboxes) {
			for (i = 0; i < checkboxes.length; i++) {
				restriction.push($(checkboxes[i]).data('nodeid').toString());
			}
		}

		nodeRestriction[groupId] = restriction;
		updateAssignNodeIcons();
		
		// write data
		$store.val(JSON.stringify(nodeRestriction));
	}

	/**
	 * Show or hide the OK button of the node restriction overlay dialog according to the validity of the selection of nodes.
	 * Call whenever the selection of nodes is changed.
	 */
	function updateOKButton() {
		var groupid = $('#channels').data('groupid')
		var initialEmpty = $.inArray(groupid.toString(), settings.initialGroups) !==-1 &&
			(typeof(settings.initialNodeRestrictions[groupid]) === "undefined" || settings.initialNodeRestrictions[groupid].length === 0);
		var somethingChecked = $("#overlay #channels input:checkbox:checked").length > 0;
		var valid = initialEmpty || !settings.restrictionRequired || somethingChecked;
		$("#overlay #overlayOK").css("visibility", valid ? "visible" : "hidden");
	}

	/**
	 * show the overlay dialog for a specific group
	 * 
	 * @param groupId id of the group the dialog should be shown for
	 */
	function showDialog(groupId) {
		var i,
			html = '',
			checked,
			restriction = nodeRestriction[groupId] ? nodeRestriction[groupId] : [],
			visible = [],
			invisible = [],
			folder,
			nodeId,
			invisible_i18n,
			icon;

		for (i = 0; i < folders.length; i++) {
			folder = folders[i];
			icon = settings.icoDomain;
			if (folder.type === 'channel') {
				icon = settings.icoDomainChannel;
			}
			checked = ($.inArray(folder.nodeId.toString(), restriction) === -1) ? '' : ' checked="checked"';
			visible.push(folder.nodeId.toString());
			html += '<li><input type="checkbox" ' +
				'data-nodeid="' + folder.nodeId + '"' + 
				checked +
				'> ' +
				'<img src="' + icon + '" /> ' + 
				folder.name +
				'</li>';
		}

		// also add the invisible nodes, that are "checked"
		for (i = 0; i < restriction.length; i++) {
			nodeId = restriction[i];
			if ($.inArray(nodeId, visible) === -1) {
				invisible.push(nodeId);
			}
		}

		if (invisible.length > 0) {
			invisible_i18n = invisible.length == 1 ? 'invisible_node_i18n' : 'invisible_nodes_i18n';
			html = '<li><input type="checkbox" checked="checked" disabled="disabled"> '
					+ '<img src="'
					+ settings.icoDomain
					+ '" /> <i>'
					+ invisible.length
					+ ' '
					+ $('#' + invisible_i18n).html() + '</i></li>' + html;
		}

		$('#channels').data('invisible', invisible);
		$('#channels').data('groupid', groupId).html(html);
		$('#overlay').fadeIn('fast');
		$('#overlay input:checkbox').change(updateOKButton);

		updateOKButton();
	}

	/**
	 * hide the overlay
	 * 
	 * @param storeData {boolean} true if changes should be stored (OK) false, otherwise (Cancel)
	 */
	function hideDialog(storeData) {
		$('#overlay').fadeOut('fast');
		
		if (storeData) {
			store();
		}
	}

	/**
	 * Auto select all nodes on newly added groups, if necessary, and store the change in $store
	 * @param $groupcb the checkbox element that has been newly checked
	 */
	function autoSelectNodes($groupcb) {
		if (settings.restrictionRequired) {
			var groupid = $groupcb.parent().parent().find('.assignNodes').data('groupid');

			// true if the group's restrictions have been deemed invalid by the server, let the user correct it
			var inValidation = $.inArray(groupid, settings.validationGroups) !== -1;

			// true if the group has already been unrestricted before the edit action started
			var initialEmpty = $.inArray(groupid.toString(), settings.initialGroups) !==-1 &&
				(typeof(settings.initialNodeRestrictions[groupid]) === "undefined" || settings.initialNodeRestrictions[groupid].length === 0);

			if (inValidation || initialEmpty) {
				return;
			}
			if ($.inArray(groupid, settings.initialGroups)===-1) {
				if (typeof nodeRestriction[groupid] === 'undefined' || nodeRestriction[groupid].length === 0) {
					nodeRestriction[groupid] = [];
					for (i = 0; i < folders.length; i++) {
						folder = folders[i];
						nodeRestriction[groupid].push(folder.nodeId.toString());
					}
					$store.val(JSON.stringify(nodeRestriction));
				}
			}
		}
	}

	/*
	 * initialisation
	 */
	
	// attach events
	$('.assignNodes').click(function () {
		var groupId = $(this).data('groupid');
		showDialog(groupId);
	});
	
	$('#overlayOK').click(function () {
		hideDialog(true);
	});
	
	$('#overlayCancel').click(function () {
		hideDialog(false);
	});

	$('div.dialog input[type=checkbox]').change(function () {
		autoSelectNodes($(this));
		updateAssignNodeIcons();
	});

	// load list of folders
	$.get('/CNPortletapp/rest/folder/getFolders/0?sid=' + settings.sid, function (data) {
		if (data.folders) {
			folders = data.folders;
			$('div.dialog input[type=checkbox]:checked').each(function(idx,el){
				autoSelectNodes($(el));
			});
			updateAssignNodeIcons();
		}
	});
};