<?
class CNPartType extends BaseObject {

 var $m_parttype;
 var $m_name;

 function CNPartType($parttype_id, $name) {
 parent::BaseObject();
 $this->m_parttype = $parttype_id;
 $this->m_name = $name;
 }

 function getPartType() {
 return $this->m_parttype;
 }

 function getName() {
 return $this->m_name;
 }

 function buildCode( &$value, $code, &$baseObj ) {
 }

 function needsClosingTag() { return false; }

 function& getEditFormComponent( &$value ) {
 }

 function storeFormToValue( &$formComponent, &$value ) {
 }

 function cleanupParts($partId) {
 }
}

class CNPartCode extends BaseObject {
 
 var $props;
 
 function CNPartCode($props=array()) {
 parent::BaseObject();
 $this->props = $props;
 }

 function buildCode( $editMode ) {
 }

 function getPartProperties() {
 return $this->props;
 }
 
 function getPropertyNames() 
 {
 $ret = array();
 foreach ( $this->props as $key=>$prop )
 {
 $ret[$key] = $key;
 }
 return $ret;
 }
 
 function &getProperty($key)
 {
 if ( isset( $this->props[$key] ) )
 return new StringObject( $this->props[$key] );
 return new NodeException( "CNPartCode: Property ($key) does not exists" );
 }
}

class CNSimplePartCode extends CNPartCode {
 
 var $text;
 
 function CNSimplePartCode($text, $props=array()) {
 parent::CNPartCode($props);
 $this->text = $text;
 }

 function buildCode( $editMode ) {
 return $this->text;
 }
 
}

class CNListPartCode extends CNPartCode {
 
 var $text;
 var $lines;
 
 function CNListPartCode($text, $lines) {
 parent::CNPartCode(array("count"=>$lines));
 $this->text = $text;
 $this->lines = $lines;
 }

 function buildCode( $editMode ) {
 return $this->text;
 }

}

class CNPageUrlPartCode extends CNPartCode {

 var $page_id;
 var $node_id;

 function CNPageUrlPartCode($page_id, $node_id, $props=array()) {
 parent::CNPartCode($props);
 $this->page_id = $page_id;
 $this->node_id = $node_id;
 }

 function buildCode( $editMode ) {
 if (!$this->page_id) return "";
 return make_link("page",$this->page_id,$node_id->toString(), ($editMode == CN_EM_PUBLISH)?"stat":"dyn");
 }


}

class CNFileUrlPartCode extends CNPartCode {

 var $file_id;
 var $type_id;
 var $node_id;

 function CNFileUrlPartCode($file_id, $node_id, $type_id, $props=array()) {
 parent::CNPartCode($props);
 $this->file_id = $file_id;
 $this->type_id = $type_id;
 $this->node_id = $node_id;
 }

 function buildCode( $editMode ) {
 if (!$this->file_id) return "";

 $val = make_link("file",$this->file_id, $this->node_id, ($editmode == CN_EM_PUBLISH)?"stat":"dyn");
 if ($this->type_id == 14) $val = str_replace("%","",rawurlencode($val));

 return $val;
 }
 
 
 function getFilePartProperties($fileId, $objId, $objType, $isType6) {
 global $T_IMAGE;
 
 if (!$fileId) return array();
 $file = get_fileinfo($fileId);
 $part_prop["id"] = $file["id"];
 $part_prop["name"] = $file["name"];
 $part_prop["description"] = $file["description"];
 $part_prop["sizeb"] = $file["filesize"];
 $part_prop["sizeKB"] = number_format (ceil($file["filesize"]*10/1024)/10, 1);
 $part_prop["sizekb"] = $part_prop["sizeKB"];
 $part_prop["sizeMB"] = number_format (ceil($file["filesize"]*10/1024/1024)/10, 1);
 $part_prop["sizemb"] = $part_prop["sizeMB"];
 $part_prop["folder_id"] = $file["folder_id"];
 $part_prop["edittimestamp"] = $file["edate"];
 $part_prop["extension"] = strtolower (eregi_replace (".*\.(.*)$", "\\1", $file["name"])); 
 if ($part_prop["sizemb"] > 1) $part_prop["size"] = $part_prop["sizemb"]." MB";
 else $part_prop["size"] = $part_prop["sizekb"]." KB";

 if ($isType6) {
 
 
 $tmp = Array();
 $tmp["id"] = $fileId;
 $tmp["type"] = $T_IMAGE;
 
 $imgtags = obj_translate( $tmp,"obj",$objId, $objType );
 while (list($k,$v) = each($imgtags)) {
 $part_prop[$k] = $v["value"];
 }
 }

 return $part_prop;
 }

}

class CNFolderUrlPartCode extends CNPartCode {

 var $folder_id;

 function CNFolderUrlPartCode($folder_id,$props=array()) {
 parent::CNPartCode($props);
 $this->folder_id = $folder_id;
 }

 function buildCode( $editMode, $code, &$baseObj ) {
 if (!$this->folder_id) return "";
 $node_id =& resolveProperty("folder.node.id", $baseObj);
 return make_link("folder",$this->folder_id,$node_id->toString(), ($editmode == CN_EM_PUBLISH)?"stat":"dyn");
 }
 

}


class CNTextPartType extends CNPartType {

 function CNTextPartType() {
 parent::CNPartType(1, i18n("Text"));
 }

 function buildCode( &$value, $code, &$baseObj ) {
 return new CNSimplePartCode(mynl2br($value->getText()));
 }

 function& getEditFormComponent( &$value )
 {
 $part = &$value->getPartObject();
 $formElement = new TextAreaComponent( $part->getName(), true, $value->getText(), REG_TEXT, !$part->isRequired(), array(5, 10, 20), array(50, 80));
 
 return $formElement;
 }
 
 function storeFormToValue( &$formComponent, &$value )
 {
 $value->setText( $formComponent->getText() );
 }
}


class CNTextHtmlPartType extends CNPartType {

 function CNTextHtmlPartType() {
 parent::CNPartType(2, i18n("Text/HTML"));
 }

 function buildCode( &$value, $code, &$baseObj ) {
 return new CNSimplePartCode(textnl2br($value->getText()));
 }
 
 function& getEditFormComponent( &$value )
 {
 $part = &$value->getPartObject();
 $formElement = new TextAreaComponent( $part->getName(), true, $value->getText(), REG_TEXT, !$part->isRequired(), array(5, 10, 20), array(50, 80));
 
 return $formElement;
 }
 
 function storeFormToValue( &$formComponent, &$value )
 {
 $value->setText( $formComponent->getText() );
 }
}


class CNHtmlPartType extends CNPartType {

 function CNHtmlPartType() {
 parent::CNPartType(3, i18n("HTML"));
 }

 function buildCode( &$value, $code, &$baseObj ) {
 return new CNSimplePartCode($value->getText());
 }
 
 function& getEditFormComponent( &$value )
 {
 $part = &$value->getPartObject();
 $formElement = new TextAreaComponent( $part->getName(), true, $value->getText(), REG_TEXT, !$part->isRequired(), array(5, 10, 20), array(50, 80));
 
 return $formElement;
 }
 
 function storeFormToValue( &$formComponent, &$value )
 {
 $value->setText( $formComponent->getText() );
 }
}


class CNUrlPagePartType extends CNPartType {

 function CNUrlPagePartType() {
 parent::CNPartType(4, i18n("URL (page)"));
 }

 function getPagePartProperties($pageId) {
 if (!$this->page_id) return array();
 $page = get_pageinfo($pageId);
 $part_prop["name"] = $page["name"];
 $part_prop["description"] = $page["description"];
 $part_prop["filename"] = $page["filename"];
 $part_prop["id"] = $page["id"];
 $part_prop["folder_id"] = $page["folder_id"];
 $part_prop["priority"] = $page["priority"];

 
 
 
 return $part_prop;
 }
 
 
 function buildCode( &$value, $code, &$baseObj ) {
 $val_ref = $value->getReference();
 $info = $value->getInfo();
 $val_text = $value->getText();
 if ($val_ref > 0 && $info == 1) {
 $node_id =& resolveProperty("folder.node.id", $baseObj);
 $class = new CNPageUrlPartCode($val_ref, $node_id->toString(), $this->getPagePartProperties($val_ref));
 } else if ($val_text && $info == 0) {
 $class = new CNSimplePartCode($val_text);
 } else {
 $class = new CNSimplePartCode("");
 }
 
 return $class;
 }

 function& getEditFormComponent( &$value )
 {
 node_class( "componentchooser" );
 node_class( "pageselector" );
 node_class( "minibrowserhiselector" );
 $part = &$value->getPartObject();
 $chooser = new ComponentChooser( $part->getName() );
 $miniComponent = new MinibrowserHiSelector( new PageSelector(), i18n("Intern") );
 $chooser->setComponent( "ref", $miniComponent );
 $chooser->setComponent( "text", new TextComponent(i18n("Extern"), true, $value->getText(), REG_TEXT_SHORT, 20, !$part->isRequired()) );
 
 $miniComponent->setObject( new CNPageObject ( $value->getReference() ) );
 $chooser->setSelection( ($value->getInfo() == 0)?"ref":"text" );
 return $chooser;
 }
 
 function storeFormToValue( &$formComponent, &$value )
 {
 $miniComponent = &$formComponent->getComponent( "ref" );
 $pageComponent = &$miniComponent->getResultObject();
 $textComponent = &$formComponent->getComponent( "text" );
 $value->setText( $textComponent->getText() );
 $value->setReference( $pageComponent->getID() );
 $value->setInfo( ($formComponent->getSelection() == "ref")?0:1 );
 }
}


class CNTagGlobalPartType extends CNPartType {

}


class CNUrlImagePartType extends CNPartType {

 function CNUrlImagePartType() {
 parent::CNPartType(6, i18n("URL (Image)"));
 }
 

 function buildCode( &$value, $code, &$baseObj ) {
 $fid = $value->getReference();
 $node_id =& resolveProperty("folder.node.id", $baseObj);
 return new CNFileUrlPartCode($fid, $node_id->toString(), 6, 
 CNFileUrlPartCode::getFilePartProperties($fid,$baseObj->getID(),
 $baseObj->getTType(),TRUE) );
 }

 function& getEditFormComponent( &$value )
 {
 node_class( "imageselector" );
 node_class( "minibrowserhiselector" );
 $part = &$value->getPartObject();
 $miniComponent = new MinibrowserHiSelector( new ImageSelector(), $part->getName() );
 $miniComponent->setObject( new CNImageObject ( $value->getReference() ) );
 return $miniComponent;
 }
 
 function storeFormToValue( &$formComponent, &$value )
 {
 $fileComponent = $formComponent->getResultObject();
 $value->setReference( $fileComponent->getID() );
 }

}


class CNUrlFilePartType extends CNPartType {

 function CNUrlFilePartType() {
 parent::CNPartType(8, i18n("URL (File)"));
 }

 function buildCode( &$value, $code, &$baseObj ) {
 $fid = $value->getReference();
 $node_id =& resolveProperty("folder.node.id", $baseObj);
 return new CNFileUrlPartCode($fid, $node_id->toString(), 8,
 CNFileUrlPartCode::getFilePartProperties($fid,$baseObj->getID(),
 $baseObj->getTType(),FALSE) );
 }

 function& getEditFormComponent( &$value )
 {
 node_class( "fileselector" );
 node_class( "minibrowserhiselector" );
 $part = &$value->getPartObject();
 $miniComponent = new MinibrowserHiSelector( new FileSelector(), $part->getName() );
 $miniComponent->setObject( new CNFileObject ( $value->getReference() ) );
 return $miniComponent;
 }
 
 function storeFormToValue( &$formComponent, &$value )
 {
 $fileComponent = $formComponent->getResultObject();
 $value->setReference( $fileComponent->getID() );
 }

}


class CNTextShortPartType extends CNPartType {

 function CNTextShortPartType() {
 parent::CNPartType(9, i18n("Text (short)"));
 }

 function buildCode( &$value, $code, &$baseObj ) {
 return new CNSimplePartCode($value->getText());
 }

 function& getEditFormComponent( &$value )
 {
 $part = &$value->getPartObject();
 $formElement = new TextComponent( $part->getName(), true, $value->getText(), REG_TEXT_SHORT, 20, !$part->isRequired() );
 
 return $formElement;
 }
 
 function storeFormToValue( &$formComponent, &$value )
 {
 $value->setText( $formComponent->getText() );
 }
}


class CNTextHtmlLongPartType extends CNPartType {

 function CNTextHtmlLongPartType() {
 parent::CNPartType(10, i18n("Text/HTML (long)"));
 }

 function buildCode( &$value, $code, &$baseObj ) {
 return new CNSimplePartCode($value->getText());
 }

 function& getEditFormComponent( &$value )
 {
 $part = &$value->getPartObject();
 $formElement = new TextAreaComponent( $part->getName(), true, $value->getText(), REG_TEXT, !$part->isRequired(), array(5, 10, 20), array(50, 80));
 
 return $formElement;
 }
 
 function storeFormToValue( &$formComponent, &$value )
 {
 $value->setText( $formComponent->getText() );
 }
}


class CNTagPagePartType extends CNPartType {

}


class CNTablePartType extends CNPartType {

}


class CNOverviewPartType extends CNPartType {

 function CNOverviewPartType() {
 parent::CNPartType(13, "Overview");
 }

 function needsClosingTag() {
 return true;
 }

 function& getEditFormComponent( &$value )
 {
 $part = &$value->getPartObject();
 $formElement = new TextComponent( $part->getName(), true, $value->getText(), REG_TEXT_SHORT, 20, !$part->isRequired() );
 
 return $formElement;
 }
 
 function storeFormToValue( &$formComponent, &$value )
 {
 $value->setText( $formComponent->getText() );
 }
}


class CNUrlFileEncPartType extends CNPartType {

 function CNUrlFileEncPartType() {
 parent::CNPartType(14, i18n("URL (File, encoded)"));
 }

 function buildCode( &$value, $code, &$baseObj ) {
 $fid = $value->getReference();
 $node_id =& resolveProperty("folder.node.id", $baseObj);
 return new CNFileUrlPartCode($fid, $node_id->toString(), 14,
 CNFileUrlPartCode::getFilePartProperties($fid,$baseObj->getID(),
 $baseObj->getTType(),FALSE) );
 }
 
 function& getEditFormComponent( &$value )
 {
 node_class( "fileselector" );
 node_class( "minibrowserhiselector" );
 $part = &$value->getPartObject();
 $miniComponent = new MinibrowserHiSelector( new FileSelector(), $part->getName() );
 $miniComponent->setObject( new CNFileObject ( $value->getReference() ) );
 return $miniComponent;
 }
 
 function storeFormToValue( &$formComponent, &$value )
 {
 $fileComponent = $formComponent->getResultObject();
 $value->setReference( $fileComponent->getID() );
 }

}


class CNListPartType extends CNPartType {

 var $listInfo;
 
 function CNListPartType($listInfo = false, $type_id = 15, $label = NULL) {
 if ($label === NULl) $label = i18n("List");
 parent::CNPartType($type_id, $label);
 $this->listInfo = $listInfo;
 }

 function buildCode( &$value, $code, &$baseObj ) {
 $info = ($this->listInfo === false)?$value->getInfo():$this->listInfo;
 if ($info == 0) $l = "ul";
 else $l = "ol";
 
 $html = "<$l>\n";
 $val_text = str_replace("\r","",$value->getText());
 $lines = explode("\n",$val_text);
 if (is_array($lines)) {
 $firstnl = true;
 $_i = 0;
 while (list($k,$v) = each($lines)) {
 if ($v) {
 $i++;
 $html .= "<li>$v</li>\n";
 $firstnl = true;
 } else {
 if ($firstnl) {
 $html .= "<br><br>\n";
 $firstnl = false;
 } else $html .= "<br>\n";
 }
 }
 }
 $html .= "</$l>\n";
 return new CNListPartCode($html, $i);
 }

 function& getEditFormComponent( &$value )
 {
 $part = &$value->getPartObject();
 $formElement = new TextAreaComponent( $part->getName(), true, $value->getText(), REG_TEXT, !$part->isRequired(), array(5, 10, 20), array(50, 80));
 
 return $formElement;
 }
 
 function storeFormToValue( &$formComponent, &$value )
 {
 $value->setText( $formComponent->getText() );
 }
}


class CNListUnsortedPartType extends CNListPartType {
 
 function CNListUnsortedPartType() {
 parent::CNListPartType(0, 16, i18n("List (Unordered)"));
 }
}


class CNListSortedPartType extends CNListPartType {

 function CNListSortedPartType() {
 parent::CNListPartType(1, 17, i18n("List (Ordered)"));
 }
}


class CNSelectImgHeightPartType extends CNPartType {

 function CNSelectImgHeightPartType() {
 parent::CNPartType(18, i18n("Select (image-height)"));
 }

 function buildCode( &$value, $code, &$baseObj ) {
 return new CNSimplePartCode($value->getText());
 }

 function& getEditFormComponent( &$value )
 {
 node_class('selectcomponent');
 $part = &$value->getPartObject();
 $formElement = new SelectComponent( $part->getName(), array( '70', '120', '180' ) );
 return $formElement;
 }
 
 function storeFormToValue( &$formComponent, &$value )
 {
 $value->setText( $formComponent->getSelection() );
 }
}


class CNSelectImgWidthPartType extends CNPartType {

 function CNSelectImgWidthPartType() {
 parent::CNPartType(19, i18n("Select (image-width)"));
 }

 function buildCode( &$value, $code, &$baseObj ) {
 return new CNSimplePartCode($value->getText());
 }

 function& getEditFormComponent( &$value )
 {
 $part = &$value->getPartObject();
 $formElement = new SelectComponent( $part->getName(), array( '110', '160', '220' ) );
 return $formElement;
 }
 
 function storeFormToValue( &$formComponent, &$value )
 {
 $value->setText( $formComponent->getSelection() );
 }
}


class CNTagTemplatePartType extends CNPartType {

}


class CNHtmlLongPartType extends CNPartType {

 function CNHtmlLongPartType() {
 parent::CNPartType(21, i18n("HTML (long)"));
 }

 function buildCode( &$value, $code, &$baseObj ) {
 return new CNSimplePartCode($value->getText());
 }
 function& getEditFormComponent( &$value )
 {
 $part = &$value->getPartObject();
 $formElement = new TextAreaComponent( $part->getName(), true, $value->getText(), REG_TEXT, !$part->isRequired(), array(5, 10, 20), array(50, 80));
 
 return $formElement;
 }
 
 function storeFormToValue( &$formComponent, &$value )
 {
 $value->setText( $formComponent->getText() );
 }
}


class CNFileLocalpathPartType extends CNPartType {

 function CNFileLocalpathPartType() {
 parent::CNPartType(22, i18n("File (localpath)"));
 }

 function buildCode( &$value, $code, &$baseObj ) {
 $val = "file://".str_replace("\\","/",substr($value->getText(),2));
 return new CNSimplePartCode($val);
 }

 function& getEditFormComponent( &$value )
 {
 $part = &$value->getPartObject();
 $formElement = new TextComponent( $part->getName(), true, $value->getText(), REG_TEXT_SHORT, 20, !$part->isRequired() );
 
 return $formElement;
 }
 
 function storeFormToValue( &$formComponent, &$value )
 {
 $value->setText( $formComponent->getText() );
 }
}


class CNTableExtPartType extends CNPartType {

}


class CNSelectClassPartType extends CNPartType {

 function CNSelectClassPartType() {
 parent::CNPartType(24, i18n("Select (class)"));
 }

 function buildCode( &$value, $code, &$baseObj ) {
 return new CNSimplePartCode($value->getText());
 }
 
 function& getEditFormComponent( &$value )
 {
 $part = &$value->getPartObject();
 $size["zellehell"] = i18n("Hell");
 $size["zelledunkel"] = i18n("Dunkel");
 $size["headernegativ"] = i18n("berschrift");
 $size["farblos"] = i18n("Farblos");
 $formElement = new SelectComponent( $part->getName(), $size );
 return $formElement;
 }
 
 function storeFormToValue( &$formComponent, &$value )
 {
 $value->setText( $formComponent->getSelection() );
 }
}


class CNUrlFolderPartType extends CNPartType {

 function CNUrlFolderPartType() {
 parent::CNPartType(8, i18n("URL (Folder)"));
 }
 
 function getFolderPartProperties($folderId) {
 if (!$folderId) return array();
 $folder = get_folderinfo($folderId);
 $part_prop["name"] = $folder["name"];
 $part_prop["description"] = $folder["description"];
 $part_prop["mother"] = $folder["mother"];
 $part_prop["node_id"] = $folder["node_id"];
 $part_prop["id"] = $folderId;
 return $part_prop;
 }

 function buildCode( &$value, $code, &$baseObj ) {
 $folderId = $value->getReference();
 return new CNFolderUrlPartCode($folderId, 
 $this->getFolderPartProperties($folderId) );
 }
 
 function& getEditFormComponent( &$value )
 {
 global $T_CONTENT, $T_FOLDER, $T_NODE;
 node_class( "minibrowserhiselector" );
 node_class( "cnfolder" );
 $part = &$value->getPartObject();
 $wrapper = new MinibrowserSimpleTree( 1, $T_CONTENT );
 $wrapper->setTType( $T_FOLDER, new CNFolderObject() );
 $wrapper->setTType( $T_NODE, new CNFolderNodeObject() );
 return new MinibrowserHiSelector( $wrapper, $part->getName() );
 }
 
 function storeFormToValue( &$formComponent, &$value )
 {
 $folderComponent = $formComponent->getResultObject();
 $value->setReference( $folderComponent->getID() );
 }

}

class CNJavaEditorPartType extends CNPartType {

 function CNJavaEditorPartType() {
 parent::CNPartType(26, i18n("Java Editor"));
 }

 function buildCode( &$value, $code, &$baseObj ) {
 return new CNSimplePartCode($value->getText());
 }

 function& getEditFormComponent( &$value )
 {
 $part = &$value->getPartObject();
 $formElement = new TextComponent( $part->getName(), true, $value->getText(), REG_TEXT_SHORT, 20, !$part->isRequired() );
 
 return $formElement;
 }
 
 function storeFormToValue( &$formComponent, &$value )
 {
 $value->setText( $formComponent->getText() );
 }
}


class CNDHtmlEditorPartType extends CNPartType {

 function CNJavaEditorPartType() {
 parent::CNPartType(27, i18n("DHTML Editor"));
 }

 function buildCode( &$value, $code, &$baseObj ) {
 return new CNSimplePartCode($value->getText());
 }

 function& getEditFormComponent( &$value )
 {
 $part = &$value->getPartObject();
 $formElement = new TextComponent( $part->getName(), true, $value->getText(), REG_TEXT_SHORT, 20, !$part->isRequired() );
 
 return $formElement;
 }
 
 function storeFormToValue( &$formComponent, &$value )
 {
 $value->setText( $formComponent->getText() );
 }
}





class CNSelectPartType extends CNPartType {
 function CNSelectPartType($parttype_id, $name) {
 parent::CNPartType($parttype_id, $name);
 }
 
 function needsClosingTag() {
 return true;
 }

 function& getEditFormComponent( &$value )
 {
 $part = &$value->getPartObject();
 $formElement = new TextComponent( $part->getName(), true, $value->getText(), REG_TEXT_SHORT, 20, !$part->isRequired() );
 
 return $formElement;
 }
 
 function storeFormToValue( &$formComponent, &$value )
 {
 $value->setText( $formComponent->getText() );
 }
 
 










 function cleanupParts($partId) {
 
 $sql = "SELECT datasource_value.dsid
			FROM datasource_value, part
			WHERE datasource_value.datasource_id = part.info_int
			AND part.id = ".sqlf_tl_num($partId); 
 $res = query($sql);
 
 
 if ($res['num'] == 0) {
 $sql = "UPDATE value
				SET value_text = ''
				WHERE part_id = ".sqlf_tl_num($partId);
 query($sql);
 return;
 }
 
 
 while (list($key,$value) = each($res['arr'])) {
 $dsValues[] = $value['dsid'];
 }
 
 
 $sql = "SELECT id, value_text
		  FROM value
			WHERE part_id = ".sqlf_tl_num($partId);
 $res = query($sql);
 if ($res['num'] == 0) return; 
 $partValues = $res['arr'];
 
 
 foreach ($partValues as $partValue) {
 if (strlen($partValue['value_text']) == 0) continue;
 $values = explode('|-|', $partValue['value_text']);
 while (list($key,$value) = each($values)) {
 if (!in_array($value,$dsValues)) unset($values[$key]);
 }
 $cleanValues = implode('|-|',$values);
 
 if (strcmp($partValue['value_text'],$cleanValues) !== 0) {
 $sql = "UPDATE value SET value_text = \"".sqlf_escaped($cleanValues)."\" WHERE id = ".sqlf_tl_num("{$partValue['id']}");
 query($sql);
 }
 }
 } 
}


class CNSelectSinglePartType extends CNSelectPartType {
 function CNSelectSinglePartType() {
 parent::CNSelectPartType(29, "Select (single)");
 }
}


class CNSelectMultiplePartType extends CNSelectPartType {
 function CNSelectMultiplePartType() {
 parent::CNSelectPartType(30, "Select (multiple)");
 }
}


class CNCheckboxPartType extends CNPartType {

 function CNCheckboxPartType() {
 parent::CNPartType(31, "Checkbox");
 }

 function buildCode( &$value, $code, &$baseObj ) {
 return new CNSimplePartCode($value->getText());
 }
 
 function& getEditFormComponent( &$value )
 {
 $part = &$value->getPartObject();
 $formElement = new CheckBoxComponent( $part->getName() ); 
 return $formElement;
 }
 
 function storeFormToValue( &$formComponent, &$value )
 {
 $value->setReference( ($formComponent->isChecked())?1:0 );
 }
}


class CNDataSourcePartType extends CNPartType {

 function CNDataSourcePartType() {
 parent::CNPartType(32, "DataSource");
 }

 function needsClosingTag() {
 return true;
 }

 function& getEditFormComponent( &$value )
 {
 $part = &$value->getPartObject();
 $formElement = new TextComponent( $part->getName(), true, $value->getText(), REG_TEXT_SHORT, 20, !$part->isRequired() );
 
 return $formElement;
 }
 
 function storeFormToValue( &$formComponent, &$value )
 {
 $value->setText( $formComponent->getText() );
 }
}
?>
